@@ -30,6 +30,7 @@ from six.moves import http_client as http
 import six.moves.urllib.parse as urlparse
 import webob.exc
 
+from glance.api import authorization
 from glance.api import common
 from glance.api import policy
 from glance.common import exception
@@ -40,7 +41,7 @@ from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.gateway
-from glance.i18n import _, _LW
+from glance.i18n import _, _LI, _LW
 import glance.notifier
 import glance.schema
 
@@ -105,12 +106,17 @@ class ImagesController(object):
         task_repo = self.gateway.get_task_repo(req.context)
         import_method = body.get('method').get('name')
         uri = body.get('method').get('uri')
+        all_stores_must_succeed = body.get('all_stores_must_succeed', True)
 
         try:
             image = image_repo.get(image_id)
-            if image.status == 'active':
+            if image.status == 'active' and import_method != "copy-image":
                 msg = _("Image with status active cannot be target for import")
                 raise exception.Conflict(msg)
+            if image.status != 'active' and import_method == "copy-image":
+                msg = _("Only images with status active can be targeted for "
+                        "copying")
+                raise exception.Conflict(msg)
             if image.status != 'queued' and import_method == 'web-download':
                 msg = _("Image needs to be in 'queued' state to use "
                         "'web-download' method")
@@ -126,33 +132,64 @@ class ImagesController(object):
             if not getattr(image, 'disk_format', None):
                 msg = _("'disk_format' needs to be set before import")
                 raise exception.Conflict(msg)
+            if not authorization.is_image_mutable(req.context, image):
+                raise webob.exc.HTTPForbidden(
+                    explanation=_("Operation not permitted"))
 
-            backend = None
+            stores = [None]
             if CONF.enabled_backends:
-                backend = req.headers.get('x-image-meta-store',
-                                          CONF.glance_store.default_backend)
                 try:
-                    glance_store.get_store_from_store_identifier(backend)
-                except glance_store.UnknownScheme:
-                    msg = _("Store for scheme %s not found") % backend
-                    LOG.warn(msg)
-                    raise exception.Conflict(msg)
+                    stores = utils.get_stores_from_request(req, body)
+                except glance_store.UnknownScheme as exc:
+                    LOG.warn(exc.msg)
+                    raise exception.Conflict(exc.msg)
+
+            # NOTE(abhishekk): If all_stores is specified and import_method is
+            # copy_image, then remove those stores where image is already
+            # present.
+            all_stores = body.get('all_stores', False)
+            if import_method == 'copy-image' and all_stores:
+                for loc in image.locations:
+                    existing_store = loc['metadata']['store']
+                    if existing_store in stores:
+                        LOG.debug("Removing store '%s' from all stores as "
+                                  "image is already available in that "
+                                  "store." % existing_store)
+                        stores.remove(existing_store)
+
+                if len(stores) == 0:
+                    LOG.info(_LI("Exiting copying workflow as image is "
+                                 "available in all configured stores."))
+                    return image_id
+
+            # validate if image is already existing in given stores when
+            # all_stores is False
+            if import_method == 'copy-image' and not all_stores:
+                for loc in image.locations:
+                    existing_store = loc['metadata']['store']
+                    if existing_store in stores:
+                        msg = _("Image is already present at store "
+                                "'%s'") % existing_store
+                        raise webob.exc.HTTPBadRequest(explanation=msg)
         except exception.Conflict as e:
             raise webob.exc.HTTPConflict(explanation=e.msg)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg)
 
+        if (not all_stores_must_succeed) and (not CONF.enabled_backends):
+            msg = (_("All_stores_must_succeed can only be set with "
+                     "enabled_backends %s") % uri)
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         task_input = {'image_id': image_id,
                       'import_req': body,
-                      'backend': backend}
+                      'backend': stores}
 
         if (import_method == 'web-download' and
-           not utils.validate_import_uri(uri)):
-                LOG.debug("URI for web-download does not pass filtering: %s",
-                          uri)
-                msg = (_("URI for web-download does not pass filtering: %s") %
-                       uri)
-                raise webob.exc.HTTPBadRequest(explanation=msg)
+                not utils.validate_import_uri(uri)):
+            LOG.debug("URI for web-download does not pass filtering: %s", uri)
+            msg = (_("URI for web-download does not pass filtering: %s") % uri)
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
         try:
             import_task = task_factory.new_task(task_type='api_image_import',
@@ -362,6 +399,68 @@ class ImagesController(object):
                    cinder_encryption_key_id)
             LOG.warn(msg)
 
+    @utils.mutating
+    def delete_from_store(self, req, store_id, image_id):
+        if not CONF.enabled_backends:
+            raise webob.exc.HTTPNotFound()
+        if store_id not in CONF.enabled_backends:
+            msg = (_("The selected store %s is not available on this node.") %
+                   store_id)
+            raise webob.exc.HTTPConflict(explanation=msg)
+
+        image_repo = self.gateway.get_repo(req.context)
+        try:
+            image = image_repo.get(image_id)
+        except exception.NotAuthenticated as e:
+            raise webob.exc.HTTPUnauthorized(explanation=e.msg)
+        except exception.NotFound:
+            msg = (_("Failed to find image %(image_id)s") %
+                   {'image_id': image_id})
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        if image.status != 'active':
+            msg = _("It's not allowed to remove image data from store if "
+                    "image status is not 'active'")
+            raise webob.exc.HTTPConflict(explanation=msg)
+
+        if len(image.locations) == 1:
+            LOG.debug("User forbidden to remove last location of image %s",
+                      image_id)
+            msg = _("Cannot delete image data from the only store containing "
+                    "it. Consider deleting the image instead.")
+            raise webob.exc.HTTPForbidden(explanation=msg)
+
+        try:
+            # NOTE(jokke): Here we go through the locations list and act on
+            # the first hit. image.locations.pop() will actually remove the
+            # data from the backend as well as remove the location object
+            # from the list.
+            for pos, loc in enumerate(image.locations):
+                if loc['metadata'].get('store') == store_id:
+                    image.locations.pop(pos)
+                    break
+            else:
+                msg = (_("Image %(iid)s is not stored in store %(sid)s.") %
+                       {'iid': image_id, 'sid': store_id})
+                raise exception.Invalid(msg)
+        except exception.Forbidden as e:
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+        except exception.Invalid as e:
+            raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except glance_store.exceptions.HasSnapshot as e:
+            raise webob.exc.HTTPConflict(explanation=e.msg)
+        except glance_store.exceptions.InUseByStore as e:
+            msg = ("The data for Image %(id)s could not be deleted "
+                   "because it is in use: %(exc)s" % {"id": image_id,
+                                                      "exc": e.msg})
+            LOG.warning(msg)
+            raise webob.exc.HTTPConflict(explanation=msg)
+        except Exception as e:
+            raise webob.exc.HTTPInternalServerError(
+                explanation=encodeutils.exception_to_unicode(e))
+
+        image_repo.save(image)
+
     @utils.mutating
     def delete(self, req, image_id):
         image_repo = self.gateway.get_repo(req.context)
@@ -409,7 +508,7 @@ class ImagesController(object):
         except (glance_store.Forbidden, exception.Forbidden) as e:
             LOG.debug("User not permitted to delete image '%s'", image_id)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
-        except (glance_store.NotFound, exception.NotFound) as e:
+        except (glance_store.NotFound, exception.NotFound):
             msg = (_("Failed to find image %(image_id)s to delete") %
                    {'image_id': image_id})
             LOG.warn(msg)
@@ -523,13 +622,15 @@ class ImagesController(object):
 
         val_data = self._validate_validation_data(image, value)
         # NOTE(abhishekk): get glance store based on location uri
+        updated_location = value
         if CONF.enabled_backends:
-            store_utils.update_store_in_locations(value, image.image_id)
+            updated_location = store_utils.get_updated_store_location(
+                value)
 
         try:
             # NOTE(flwang): _locations_proxy's setattr method will check if
             # the update is acceptable.
-            image.locations = value
+            image.locations = updated_location
             if image.status == 'queued':
                 for k, v in val_data.items():
                     setattr(image, k, v)
@@ -553,8 +654,10 @@ class ImagesController(object):
 
         val_data = self._validate_validation_data(image, [value])
         # NOTE(abhishekk): get glance store based on location uri
+        updated_location = value
         if CONF.enabled_backends:
-            store_utils.update_store_in_locations([value], image.image_id)
+            updated_location = store_utils.get_updated_store_location(
+                [value])[0]
 
         pos = self._get_locations_op_pos(path_pos,
                                          len(image.locations), True)
@@ -562,7 +665,7 @@ class ImagesController(object):
             msg = _("Invalid position for adding a location.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
         try:
-            image.locations.insert(pos, value)
+            image.locations.insert(pos, updated_location)
             if image.status == 'queued':
                 for k, v in val_data.items():
                     setattr(image, k, v)
@@ -607,7 +710,7 @@ class ImagesController(object):
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
-    _disallowed_properties = ('direct_url', 'self', 'file', 'schema')
+    _disallowed_properties = ('direct_url', 'self', 'file', 'schema', 'stores')
     _readonly_properties = ('created_at', 'updated_at', 'status', 'checksum',
                             'size', 'virtual_size', 'direct_url', 'self',
                             'file', 'schema', 'id', 'os_hash_algo',
@@ -1020,6 +1123,17 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             msg = _("Unknown import method name '%s'.") % method_name
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
+        # Validate 'all_stores_must_succeed' and 'all_stores'
+        all_stores_must_succeed = body.get('all_stores_must_succeed', True)
+        if not isinstance(all_stores_must_succeed, bool):
+            msg = (_("'all_stores_must_succeed' must be boolean value only"))
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        all_stores = body.get('all_stores', False)
+        if not isinstance(all_stores, bool):
+            msg = (_("'all_stores' must be boolean value only"))
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
     def import_image(self, request):
         body = self._get_request_body(request)
         self._validate_import_body(body)
@@ -1079,8 +1193,8 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
                 locations = _get_image_locations(image)
                 if locations:
                     # Choose best location configured strategy
-                    l = location_strategy.choose_best_location(locations)
-                    image_view['direct_url'] = l['url']
+                    loc = location_strategy.choose_best_location(locations)
+                    image_view['direct_url'] = loc['url']
                 else:
                     LOG.debug("The 'locations' list of image %s is empty, "
                               "not including 'direct_url' in response",
@@ -1156,6 +1270,9 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
                                                          ensure_ascii=False))
         response.content_type = 'application/json'
 
+    def delete_from_store(self, response, result):
+        response.status_int = http.NO_CONTENT
+
     def delete(self, response, result):
         response.status_int = http.NO_CONTENT
 
