@@ -16,6 +16,7 @@ import os
 
 import glance_store as store_api
 from glance_store import backend
+from glance_store import exceptions as store_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
@@ -75,6 +76,12 @@ CONF.register_opts(api_import_opts, group='image_import_opts')
 # need to duplicate what we have already for example in base_import.py.
 
 
+class _NoStoresSucceeded(exception.GlanceException):
+
+    def __init__(self, message):
+        super(_NoStoresSucceeded, self).__init__(message)
+
+
 class _DeleteFromFS(task.Task):
 
     def __init__(self, task_id, task_type):
@@ -89,7 +96,13 @@ class _DeleteFromFS(task.Task):
         :param file_path: path to the file being deleted
         """
         if CONF.enabled_backends:
-            store_api.delete(file_path, 'os_glance_staging_store')
+            try:
+                store_api.delete(file_path, 'os_glance_staging_store')
+            except store_api.exceptions.NotFound as e:
+                LOG.error(_("After upload to backend, deletion of staged "
+                            "image data from %(fn)s has failed because "
+                            "%(em)s"), {'fn': file_path,
+                                        'em': e.message})
         else:
             # TODO(abhishekk): After removal of backend module from
             # glance_store need to change this to use multi_backend
@@ -188,13 +201,16 @@ class _VerifyStaging(task.Task):
 
 class _ImportToStore(task.Task):
 
-    def __init__(self, task_id, task_type, image_repo, uri, image_id, backend):
+    def __init__(self, task_id, task_type, image_repo, uri, image_id, backend,
+                 all_stores_must_succeed, set_active):
         self.task_id = task_id
         self.task_type = task_type
         self.image_repo = image_repo
         self.uri = uri
         self.image_id = image_id
         self.backend = backend
+        self.all_stores_must_succeed = all_stores_must_succeed
+        self.set_active = set_active
         super(_ImportToStore, self).__init__(
             name='%s-ImportToStore-%s' % (task_type, task_id))
 
@@ -245,37 +261,102 @@ class _ImportToStore(task.Task):
         # will need the file path anyways for our delete workflow for now.
         # For future proofing keeping this as is.
         image = self.image_repo.get(self.image_id)
-        image_import.set_image_data(image, file_path or self.uri, self.task_id,
-                                    backend=self.backend)
-
-        # NOTE(flaper87): We need to save the image again after the locations
-        # have been set in the image.
+        if image.status == "deleted":
+            raise exception.ImportTaskError("Image has been deleted, aborting"
+                                            " import.")
+        try:
+            image_import.set_image_data(image, file_path or self.uri,
+                                        self.task_id, backend=self.backend,
+                                        set_active=self.set_active)
+        # NOTE(yebinama): set_image_data catches Exception and raises from
+        # them. Can't be more specific on exceptions catched.
+        except Exception:
+            if self.all_stores_must_succeed:
+                raise
+            msg = (_("%(task_id)s of %(task_type)s failed but since "
+                     "all_stores_must_succeed is set to false, continue.") %
+                   {'task_id': self.task_id, 'task_type': self.task_type})
+            LOG.warning(msg)
+            if self.backend is not None:
+                failed_import = image.extra_properties.get(
+                    'os_glance_failed_import', '').split(',')
+                failed_import.append(self.backend)
+                image.extra_properties['os_glance_failed_import'] = ','.join(
+                    failed_import).lstrip(',')
+        if self.backend is not None:
+            importing = image.extra_properties.get(
+                'os_glance_importing_to_stores', '').split(',')
+            try:
+                importing.remove(self.backend)
+                image.extra_properties[
+                    'os_glance_importing_to_stores'] = ','.join(
+                    importing).lstrip(',')
+            except ValueError:
+                LOG.debug("Store %s not found in property "
+                          "os_glance_importing_to_stores.", self.backend)
+        # NOTE(flaper87): We need to save the image again after
+        # the locations have been set in the image.
         self.image_repo.save(image)
 
+    def revert(self, result, **kwargs):
+        """
+        Remove location from image in case of failure
 
-class _SaveImage(task.Task):
+        :param result: taskflow result object
+        """
+        image = self.image_repo.get(self.image_id)
+        for i, location in enumerate(image.locations):
+            if location.get('metadata', {}).get('store') == self.backend:
+                try:
+                    image.locations.pop(i)
+                except (store_exceptions.NotFound,
+                        store_exceptions.Forbidden):
+                    msg = (_("Error deleting from store %{store}s when "
+                             "reverting.") % {'store': self.backend})
+                    LOG.warning(msg)
+                # NOTE(yebinama): Some store drivers doesn't document which
+                # exceptions they throw.
+                except Exception:
+                    msg = (_("Unexpected exception when deleting from store"
+                             "%{store}s.") % {'store': self.backend})
+                    LOG.warning(msg)
+                else:
+                    if len(image.locations) == 0:
+                        image.checksum = None
+                        image.os_hash_algo = None
+                        image.os_hash_value = None
+                        image.size = None
+                    self.image_repo.save(image)
+                break
 
-    def __init__(self, task_id, task_type, image_repo, image_id):
+
+class _VerifyImageState(task.Task):
+
+    def __init__(self, task_id, task_type, image_repo, image_id,
+                 import_method):
         self.task_id = task_id
         self.task_type = task_type
         self.image_repo = image_repo
         self.image_id = image_id
-        super(_SaveImage, self).__init__(
-            name='%s-SaveImage-%s' % (task_type, task_id))
+        self.import_method = import_method
+        super(_VerifyImageState, self).__init__(
+            name='%s-VerifyImageState-%s' % (task_type, task_id))
 
     def execute(self):
-        """Transition image status to active
+        """Verify we have active image
 
         :param image_id: Glance Image ID
         """
         new_image = self.image_repo.get(self.image_id)
-        if new_image.status == 'saving':
-            # NOTE(flaper87): THIS IS WRONG!
-            # we should be doing atomic updates to avoid
-            # race conditions. This happens in other places
-            # too.
-            new_image.status = 'active'
-        self.image_repo.save(new_image)
+        if new_image.status != 'active':
+            raise _NoStoresSucceeded(_('None of the uploads finished!'))
+
+    def revert(self, result, **kwargs):
+        """Set back to queued if this wasn't copy-image job."""
+        if self.import_method != 'copy-image':
+            new_image = self.image_repo.get(self.image_id)
+            new_image.status = 'queued'
+            self.image_repo.save_image(new_image)
 
 
 class _CompleteTask(task.Task):
@@ -337,13 +418,15 @@ def get_flow(**kwargs):
     image_id = kwargs.get('image_id')
     import_method = kwargs.get('import_req')['method']['name']
     uri = kwargs.get('import_req')['method'].get('uri')
-    backend = kwargs.get('backend')
+    stores = kwargs.get('backend', [None])
+    all_stores_must_succeed = kwargs.get('import_req').get(
+        'all_stores_must_succeed', True)
 
     separator = ''
     if not CONF.enabled_backends and not CONF.node_staging_uri.endswith('/'):
         separator = '/'
 
-    if not uri and import_method == 'glance-direct':
+    if not uri and import_method in ['glance-direct', 'copy-image']:
         if CONF.enabled_backends:
             separator, staging_dir = store_utils.get_dir_separator()
             uri = separator.join((staging_dir, str(image_id)))
@@ -352,9 +435,9 @@ def get_flow(**kwargs):
 
     flow = lf.Flow(task_type, retry=retry.AlwaysRevert())
 
-    if import_method == 'web-download':
-        downloadToStaging = internal_plugins.get_import_plugin(**kwargs)
-        flow.add(downloadToStaging)
+    if import_method in ['web-download', 'copy-image']:
+        internal_plugin = internal_plugins.get_import_plugin(**kwargs)
+        flow.add(internal_plugin)
         if CONF.enabled_backends:
             separator, staging_dir = store_utils.get_dir_separator()
             file_uri = separator.join((staging_dir, str(image_id)))
@@ -365,25 +448,41 @@ def get_flow(**kwargs):
 
     flow.add(_VerifyStaging(task_id, task_type, task_repo, file_uri))
 
-    for plugin in import_plugins.get_import_plugins(**kwargs):
-        flow.add(plugin)
+    # Note(jokke): The plugins were designed to act on the image data or
+    # metadata during the import process before the image goes active. It
+    # does not make sense to try to execute them during 'copy-image'.
+    if import_method != 'copy-image':
+        for plugin in import_plugins.get_import_plugins(**kwargs):
+            flow.add(plugin)
+    else:
+        LOG.debug("Skipping plugins on 'copy-image' job.")
 
-    import_to_store = _ImportToStore(task_id,
-                                     task_type,
-                                     image_repo,
-                                     file_uri,
-                                     image_id,
-                                     backend)
-    flow.add(import_to_store)
+    for idx, store in enumerate(stores, 1):
+        set_active = (not all_stores_must_succeed) or (idx == len(stores))
+        if import_method == 'copy-image':
+            set_active = False
+        task_name = task_type + "-" + (store or "")
+        import_task = lf.Flow(task_name)
+        import_to_store = _ImportToStore(task_id,
+                                         task_name,
+                                         image_repo,
+                                         file_uri,
+                                         image_id,
+                                         store,
+                                         all_stores_must_succeed,
+                                         set_active)
+        import_task.add(import_to_store)
+        flow.add(import_task)
 
     delete_task = lf.Flow(task_type).add(_DeleteFromFS(task_id, task_type))
     flow.add(delete_task)
 
-    save_task = _SaveImage(task_id,
-                           task_type,
-                           image_repo,
-                           image_id)
-    flow.add(save_task)
+    verify_task = _VerifyImageState(task_id,
+                                    task_type,
+                                    image_repo,
+                                    image_id,
+                                    import_method)
+    flow.add(verify_task)
 
     complete_task = _CompleteTask(task_id,
                                   task_type,
@@ -393,7 +492,14 @@ def get_flow(**kwargs):
 
     image = image_repo.get(image_id)
     from_state = image.status
-    image.status = 'importing'
+    if import_method != 'copy-image':
+        image.status = 'importing'
+
+    image.extra_properties[
+        'os_glance_importing_to_stores'] = ','.join((store for store in
+                                                     stores if
+                                                     store is not None))
+    image.extra_properties['os_glance_failed_import'] = ''
     image_repo.save(image, from_state=from_state)
 
     return flow
