@@ -25,6 +25,14 @@ from glance.i18n import _
 registry_addr_opts = [
     cfg.HostAddressOpt('registry_host',
                        default='0.0.0.0',
+                       deprecated_for_removal=True,
+                       deprecated_since="Queens",
+                       deprecated_reason=_("""
+Glance registry service is deprecated for removal.
+
+More information can be found from the spec:
+http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
+"""),
                        help=_("""
 Address the registry server is hosted on.
 
@@ -36,6 +44,14 @@ Related options:
 
 """)),
     cfg.PortOpt('registry_port', default=9191,
+                deprecated_for_removal=True,
+                deprecated_since="Queens",
+                deprecated_reason=_("""
+Glance registry service is deprecated for removal.
+
+More information can be found from the spec:
+http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
+"""),
                 help=_("""
 Port the registry server is listening on.
 
