@@ -15,6 +15,7 @@
 
 from unittest import mock
 
+from glance_store import exceptions as store_exceptions
 from oslo_config import cfg
 
 import glance.async_.flows.api_image_import as import_flow
@@ -166,3 +167,96 @@ class TestImportToStoreTask(test_utils.BaseTestCase):
                              "store1")
         except cursive_exception.SignatureVerificationError:
             self.fail("Exception shouldn't be raised")
+
+
+class TestDeleteFromFS(test_utils.BaseTestCase):
+    def test_delete_with_backends_deletes(self):
+        task = import_flow._DeleteFromFS(TASK_ID1, TASK_TYPE)
+        self.config(enabled_backends='file:foo')
+        with mock.patch.object(import_flow.store_api, 'delete') as mock_del:
+            task.execute(mock.sentinel.path)
+            mock_del.assert_called_once_with(
+                mock.sentinel.path,
+                'os_glance_staging_store')
+
+    def test_delete_with_backends_delete_fails(self):
+        self.config(enabled_backends='file:foo')
+        task = import_flow._DeleteFromFS(TASK_ID1, TASK_TYPE)
+        with mock.patch.object(import_flow.store_api, 'delete') as mock_del:
+            mock_del.side_effect = store_exceptions.NotFound(image=IMAGE_ID1,
+                                                             message='Testing')
+            # If we didn't swallow this we would explode here
+            task.execute(mock.sentinel.path)
+            mock_del.assert_called_once_with(
+                mock.sentinel.path,
+                'os_glance_staging_store')
+
+            # Raise something unexpected and make sure it bubbles up
+            mock_del.side_effect = RuntimeError
+            self.assertRaises(RuntimeError,
+                              task.execute, mock.sentinel.path)
+
+    @mock.patch('os.path.exists')
+    @mock.patch('os.unlink')
+    def test_delete_without_backends_exists(self, mock_unlink, mock_exists):
+        mock_exists.return_value = True
+        task = import_flow._DeleteFromFS(TASK_ID1, TASK_TYPE)
+        task.execute('1234567foo')
+        # FIXME(danms): I have no idea why the code arbitrarily snips
+        # the first seven characters from the path. Need a comment or
+        # *something*.
+        mock_unlink.assert_called_once_with('foo')
+
+        mock_unlink.reset_mock()
+        mock_unlink.side_effect = OSError(123, 'failed')
+        # Make sure we swallow the OSError and don't explode
+        task.execute('1234567foo')
+
+    @mock.patch('os.path.exists')
+    @mock.patch('os.unlink')
+    def test_delete_without_backends_missing(self, mock_unlink, mock_exists):
+        mock_exists.return_value = False
+        task = import_flow._DeleteFromFS(TASK_ID1, TASK_TYPE)
+        task.execute('foo')
+        mock_unlink.assert_not_called()
+
+
+class TestVerifyImageStateTask(test_utils.BaseTestCase):
+    def test_verify_active_status(self):
+        fake_img = mock.MagicMock(status='active')
+        mock_repo = mock.MagicMock()
+        mock_repo.get.return_value = fake_img
+
+        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
+                                             mock_repo, IMAGE_ID1,
+                                             'anything!')
+
+        task.execute()
+
+        fake_img.status = 'importing'
+        self.assertRaises(import_flow._NoStoresSucceeded,
+                          task.execute)
+
+    def test_revert_copy_status_unchanged(self):
+        fake_img = mock.MagicMock(status='active')
+        mock_repo = mock.MagicMock()
+        mock_repo.get.return_value = fake_img
+        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
+                                             mock_repo, IMAGE_ID1,
+                                             'copy-image')
+        task.revert(mock.sentinel.result)
+
+        # If we are doing copy-image, no state update should be made
+        mock_repo.save_image.assert_not_called()
+
+    def test_reverts_state_nocopy(self):
+        fake_img = mock.MagicMock(status='importing')
+        mock_repo = mock.MagicMock()
+        mock_repo.get.return_value = fake_img
+        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
+                                             mock_repo, IMAGE_ID1,
+                                             'glance-direct')
+        task.revert(mock.sentinel.result)
+
+        # Except for copy-image, image state should revert to queued
+        mock_repo.save_image.assert_called_once()
