@@ -15,9 +15,9 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 import uuid
 
-import mock
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_utils import encodeutils
@@ -61,6 +61,7 @@ class TestDbUtilities(test_utils.BaseTestCase):
         import_module.assert_called_once_with('glance.db.sqlalchemy.api')
         api.configure.assert_called_once_with()
 
+
 UUID1 = 'c80a1a6c-bd1f-41c5-90ee-81afedb1d58d'
 UUID2 = 'a85abd86-55b3-4d5b-b0b4-5d0a6e6042fc'
 UUID3 = '971ec09a-8067-4bc8-a91f-ae3557f1c4c7'
@@ -377,6 +378,7 @@ class TestImageRepo(test_utils.BaseTestCase):
         original_update_time = image.updated_at
         image.name = 'foo'
         image.tags = ['king', 'kong']
+        self.delay_inaccurate_clock()
         self.image_repo.save(image)
         current_update_time = image.updated_at
         self.assertGreater(current_update_time, original_update_time)
@@ -396,6 +398,7 @@ class TestImageRepo(test_utils.BaseTestCase):
     def test_remove_image(self):
         image = self.image_repo.get(UUID1)
         previous_update_time = image.updated_at
+        self.delay_inaccurate_clock()
         self.image_repo.remove(image)
         self.assertGreater(image.updated_at, previous_update_time)
         self.assertRaises(exception.ImageNotFound, self.image_repo.get, UUID1)
@@ -756,6 +759,7 @@ class TestTaskRepo(test_utils.BaseTestCase):
     def test_save_task(self):
         task = self.task_repo.get(UUID1)
         original_update_time = task.updated_at
+        self.delay_inaccurate_clock()
         self.task_repo.save(task)
         current_update_time = task.updated_at
         self.assertGreater(current_update_time, original_update_time)
